@@ -3,7 +3,6 @@
 #include "rime_config.h"
 #include <stdlib.h>
 #include <string.h>
-#include <dlfcn.h>
 #include <signal.h>
 #include <glib.h>
 #include <glib-object.h>
@@ -19,7 +18,7 @@
 
 #define DISTRIBUTION_NAME _("Rime")
 #define DISTRIBUTION_CODE_NAME "ibus-rime"
-#define DISTRIBUTION_VERSION RIME_VERSION
+#define DISTRIBUTION_VERSION IBUS_RIME_VERSION
 
 RimeApi *rime_api = NULL;
 
@@ -30,14 +29,6 @@ static const char* get_ibus_rime_user_data_dir(char *path) {
   return path;
 }
 
-static const char* get_ibus_rime_old_user_data_dir(char *path) {
-  const char* home = getenv("HOME");
-  strcpy(path, home);
-  strcat(path, "/.ibus/rime");
-  return path;
-}
-
-
 static void show_message(const char* summary, const char* details) {
   NotifyNotification* notice = notify_notification_new(summary, details, NULL);
   notify_notification_show(notice, NULL);
@@ -64,29 +55,24 @@ static void notification_handler(void* context_object,
   }
 }
 
+static void fill_traits(RimeTraits *traits) {
+  traits->shared_data_dir = IBUS_RIME_SHARED_DATA_DIR;
+  traits->distribution_name = DISTRIBUTION_NAME;
+  traits->distribution_code_name = DISTRIBUTION_CODE_NAME;
+  traits->distribution_version = DISTRIBUTION_VERSION;
+  traits->app_name = "rime.ibus";
+}
+
 void ibus_rime_start(gboolean full_check) {
   char user_data_dir[512] = {0};
-  char old_user_data_dir[512] = {0};
   get_ibus_rime_user_data_dir(user_data_dir);
   if (!g_file_test(user_data_dir, G_FILE_TEST_IS_DIR)) {
-    get_ibus_rime_old_user_data_dir(old_user_data_dir);
-    if (g_file_test(old_user_data_dir, G_FILE_TEST_IS_DIR)) {
-      g_rename(old_user_data_dir, user_data_dir);
-    }
-    else {
-      g_mkdir_with_parents(user_data_dir, 0700);
-    }
+    g_mkdir_with_parents(user_data_dir, 0700);
   }
-  rime_api->set_notification_handler(notification_handler, NULL);
   RIME_STRUCT(RimeTraits, ibus_rime_traits);
-  ibus_rime_traits.shared_data_dir = IBUS_RIME_SHARED_DATA_DIR;
+  fill_traits(&ibus_rime_traits);
   ibus_rime_traits.user_data_dir = user_data_dir;
-  ibus_rime_traits.distribution_name = DISTRIBUTION_NAME;
-  ibus_rime_traits.distribution_code_name = DISTRIBUTION_CODE_NAME;
-  ibus_rime_traits.distribution_version = DISTRIBUTION_VERSION;
-  ibus_rime_traits.app_name = "ibus";
-  static RIME_MODULE_LIST(ibus_rime_modules, "default", "legacy");
-  ibus_rime_traits.modules = ibus_rime_modules;
+
   rime_api->initialize(&ibus_rime_traits);
   if (rime_api->start_maintenance((Bool)full_check)) {
     // update frontend config
@@ -94,14 +80,8 @@ void ibus_rime_start(gboolean full_check) {
   }
 }
 
-static void* legacy_module_handle = NULL;
-
-static void load_plugin_modules() {
-  legacy_module_handle = dlopen("librime-legacy.so", RTLD_LAZY);
-}
-
-static void unload_plugin_modules() {
-  dlclose(legacy_module_handle);
+void ibus_rime_stop() {
+  rime_api->finalize();
 }
 
 static void ibus_disconnect_cb(IBusBus *bus, gpointer user_data) {
@@ -134,16 +114,19 @@ static void rime_with_ibus() {
     g_error("notify_init failed");
     exit(1);
   }
+  rime_api->set_notification_handler(notification_handler, NULL);
+
+  RIME_STRUCT(RimeTraits, ibus_rime_traits);
+  fill_traits(&ibus_rime_traits);
+  rime_api->setup(&ibus_rime_traits);
 
-  load_plugin_modules();
   gboolean full_check = FALSE;
   ibus_rime_start(full_check);
   ibus_rime_load_settings();
 
   ibus_main();
 
-  rime_api->finalize();
-  unload_plugin_modules();
+  ibus_rime_stop();
   notify_uninit();
 
   g_object_unref(factory);
@@ -152,7 +135,7 @@ static void rime_with_ibus() {
 
 static void sigterm_cb(int sig) {
   if (rime_api) {
-    rime_api->finalize();
+    ibus_rime_stop();
   }
   notify_uninit();
   exit(EXIT_FAILURE);
