@@ -14,9 +14,11 @@ static struct ColorSchemeDefinition preset_color_schemes[] = {
 };
 
 static struct IBusRimeSettings ibus_rime_settings_default = {
-  FALSE,
-  IBUS_ORIENTATION_SYSTEM,
-  &preset_color_schemes[0],
+  .embed_preedit_text = TRUE,
+  .preedit_style = PREEDIT_STYLE_COMPOSITION,
+  .cursor_type = CURSOR_TYPE_INSERT,
+  .lookup_table_orientation = IBUS_ORIENTATION_SYSTEM,
+  .color_scheme = NULL,
 };
 
 struct IBusRimeSettings g_ibus_rime_settings;
@@ -34,7 +36,7 @@ select_color_scheme(struct IBusRimeSettings* settings,
     }
   }
   // fallback to default
-  settings->color_scheme = &preset_color_schemes[0];
+  settings->color_scheme = NULL;
 }
 
 void
@@ -54,6 +56,26 @@ ibus_rime_load_settings()
     g_ibus_rime_settings.embed_preedit_text = !!inline_preedit;
   }
 
+  const char* preedit_style_str =
+      rime_api->config_get_cstring(&config, "style/preedit_style");
+  if(preedit_style_str) {
+    if(!strcmp(preedit_style_str, "composition")) {
+      g_ibus_rime_settings.preedit_style = PREEDIT_STYLE_COMPOSITION;
+    } else if(!strcmp(preedit_style_str, "preview")) {
+      g_ibus_rime_settings.preedit_style = PREEDIT_STYLE_PREVIEW;
+    }
+  }
+
+  const char* cursor_type_str =
+      rime_api->config_get_cstring(&config, "style/cursor_type");
+  if (cursor_type_str) {
+    if (!strcmp(cursor_type_str, "insert")) {
+      g_ibus_rime_settings.cursor_type = CURSOR_TYPE_INSERT;
+    } else if (!strcmp(cursor_type_str, "select")) {
+      g_ibus_rime_settings.cursor_type = CURSOR_TYPE_SELECT;
+    }
+  }
+
   Bool horizontal = False;
   if (rime_api->config_get_bool(&config, "style/horizontal", &horizontal)) {
     g_ibus_rime_settings.lookup_table_orientation =
